
{******************************************************************************}
{* DCPcrypt v2.0 written by David Barton (crypto@cityinthesky.co.uk) **********}
{******************************************************************************}
{* A Base64 encoding/decoding unit ********************************************}
{******************************************************************************}
{* Copyright (c) 1999-2002 David Barton                                       *}
{* Permission is hereby granted, free of charge, to any person obtaining a    *}
{* copy of this software and associated documentation files (the "Software"), *}
{* to deal in the Software without restriction, including without limitation  *}
{* the rights to use, copy, modify, merge, publish, distribute, sublicense,   *}
{* and/or sell copies of the Software, and to permit persons to whom the      *}
{* Software is furnished to do so, subject to the following conditions:       *}
{*                                                                            *}
{* The above copyright notice and this permission notice shall be included in *}
{* all copies or substantial portions of the Software.                        *}
{*                                                                            *}
{* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR *}
{* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,   *}
{* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL    *}
{* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER *}
{* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING    *}
{* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER        *}
{* DEALINGS IN THE SOFTWARE.                                                  *}
{******************************************************************************}
UNIT base64;

INTERFACE
USES
  Sysutils;

FUNCTION Base64EncodeStr(CONST Value: string): string;
  { Encode a string into Base64 format }
FUNCTION Base64DecodeStr(CONST Value: string): string;
  { Decode a Base64 format string }
FUNCTION Base64Encode(pInput: POINTER; pOutput: POINTER; Size: LONGINT): LONGINT;
  { Encode a lump of raw data (output is (4/3) times bigger than input) }
FUNCTION Base64Decode(pInput: POINTER; pOutput: POINTER; Size: LONGINT): LONGINT;
  { Decode a lump of raw data }


{******************************************************************************}
{******************************************************************************}
IMPLEMENTATION
{$Q-}{$R-}

CONST
  B64: ARRAY[0..63] OF BYTE= (65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,
    81,82,83,84,85,86,87,88,89,90,97,98,99,100,101,102,103,104,105,106,107,108,
    109,110,111,112,113,114,115,116,117,118,119,120,121,122,48,49,50,51,52,53,
    54,55,56,57,43,47);

FUNCTION Base64Encode(pInput: POINTER; pOutput: POINTER; Size: LONGINT): LONGINT;
VAR
  i, iptr, optr: INTEGER;
  Input, Output: PByteArray;
BEGIN
  Input:= PByteArray(pInput); Output:= PByteArray(pOutput);
  iptr:= 0; optr:= 0;
  FOR i:= 1 TO (Size div 3) DO
  BEGIN
    Output^[optr+0]:= B64[Input^[iptr] shr 2];
    Output^[optr+1]:= B64[((Input^[iptr] AND 3) shl 4) + (Input^[iptr+1] shr 4)];
    Output^[optr+2]:= B64[((Input^[iptr+1] AND 15) shl 2) + (Input^[iptr+2] shr 6)];
    Output^[optr+3]:= B64[Input^[iptr+2] AND 63];
    Inc(optr,4); Inc(iptr,3);
  END;
  CASE (Size mod 3) OF
    1: BEGIN
         Output^[optr+0]:= B64[Input^[iptr] shr 2];
         Output^[optr+1]:= B64[(Input^[iptr] AND 3) shl 4];
         Output^[optr+2]:= BYTE('=');
         Output^[optr+3]:= BYTE('=');
       END;
    2: BEGIN
         Output^[optr+0]:= B64[Input^[iptr] shr 2];
         Output^[optr+1]:= B64[((Input^[iptr] AND 3) shl 4) + (Input^[iptr+1] shr 4)];
         Output^[optr+2]:= B64[(Input^[iptr+1] AND 15) shl 2];
         Output^[optr+3]:= BYTE('=');
       END;
  END;
  Result:= ((Size+2) div 3) * 4;
END;

FUNCTION Base64EncodeStr(CONST Value: string): string;
BEGIN
  SetLength(Result,((Length(Value)+2) div 3) * 4);
  Base64Encode(@Value[1],@Result[1],Length(Value));
END;

FUNCTION Base64Decode(pInput: POINTER; pOutput: POINTER; Size: LONGINT): LONGINT;
VAR
  i, j, iptr, optr: INTEGER;
  Temp: ARRAY[0..3] OF BYTE;
  Input, Output: PByteArray;
BEGIN
  Input:= PByteArray(pInput); Output:= PByteArray(pOutput);
  iptr:= 0; optr:= 0;
  Result:= 0;
  FOR i:= 1 TO (Size div 4) DO
  BEGIN
    FOR j:= 0 TO 3 DO
    BEGIN
      CASE Input^[iptr] OF
        65..90 : Temp[j]:= Input^[iptr] - Ord('A');
        97..122: Temp[j]:= Input^[iptr] - Ord('a') + 26;
        48..57 : Temp[j]:= Input^[iptr] - Ord('0') + 52;
        43     : Temp[j]:= 62;
        47     : Temp[j]:= 63;
        61     : Temp[j]:= $FF;
      END;
      Inc(iptr);
    END;
    Output^[optr]:= (Temp[0] shl 2) OR (Temp[1] shr 4);
    Result:= optr+1;
    IF (Temp[2]<> $FF) AND (Temp[3]= $FF) THEN
    BEGIN
      Output^[optr+1]:= (Temp[1] shl 4) OR (Temp[2] shr 2);
      Result:= optr+2;
      Inc(optr)
    END
    ELSE IF (Temp[2]<> $FF) THEN
    BEGIN
      Output^[optr+1]:= (Temp[1] shl 4) OR (Temp[2] shr 2);
      Output^[optr+2]:= (Temp[2] shl 6) OR  Temp[3];
      Result:= optr+3;
      Inc(optr,2);
    END;
    Inc(optr);
  END;
END;

FUNCTION Base64DecodeStr(CONST Value: string): string;
BEGIN
  SetLength(Result,(Length(Value) div 4) * 3);
  SetLength(Result,Base64Decode(@Value[1],@Result[1],Length(Value)));
END;


END.
